import datetime

from flask import jsonify, Response
from concurrent.futures import ThreadPoolExecutor
from flask import Blueprint, current_app
from brian_dashboard_manager.routes import common
from brian_dashboard_manager.grafana.provision import provision
from brian_dashboard_manager import CONFIG_KEY

provision_state = {
    'time': datetime.datetime.now(datetime.timezone.utc),
    'provisioning': False
}

routes = Blueprint("update", __name__)

UPDATE_RESPONSE_SCHEMA = {
    'type': 'object',
    'properties': {
        'message': {
            'type': 'string'
        }
    }
}


@routes.after_request
def after_request(resp):
    return common.after_request(resp)


def provision_maybe():
    """
    Check if we should provision in case of multiple requests hitting the endpoint.
    We need to make sure we don't provision if another thread is still running.

    :return: tuple of (bool, datetime) representing if we can provision
        and the timestamp of the last provisioning, respectively.
    """

    global provision_state

    now = datetime.datetime.now(datetime.timezone.utc)
    timestamp = provision_state['time']
    provisioning = provision_state['provisioning']

    if provisioning and (now - timestamp).total_seconds() < 600:  # lockout for 10 minutes at most
        return False, timestamp

    def write_timestamp(timestamp, provisioning):
        provision_state['time'] = timestamp
        provision_state['provisioning'] = provisioning

    def _finish():
        now = datetime.datetime.now(datetime.timezone.utc)
        write_timestamp(now, False)

    write_timestamp(now, True)

    executor = ThreadPoolExecutor(max_workers=1)
    f = executor.submit(provision, current_app.config[CONFIG_KEY])
    f.add_done_callback(lambda _: _finish())
    return True, now


@routes.route('/', methods=['GET'])
def update():
    """
    This resource is used to trigger the provisioning to Grafana.

    It responds to the request immediately after starting
    the provisioning process.

    The response will be formatted according to the following schema:

    .. asjson::
       brian_dashboard_manager.routes.update.UPDATE_RESPONSE_SCHEMA

    :return: json
    """
    should, timestamp = provision_maybe()
    if should:
        return jsonify({'data': {'message': 'Provisioning dashboards!'}})
    else:
        seconds_ago = (datetime.datetime.now(datetime.timezone.utc) - timestamp).total_seconds()
        message = f'Provision already in progress since {timestamp} ({seconds_ago:.2f} seconds ago).'
        return Response(message, status=503)
